/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT2_STUBS_API
#define ORBIT_IDL_C_COMMON
#define Foo_COMMON
#include "Foo.h"

static const CORBA_unsigned_long ORBit_zero_int = 0;
#if ( (TC_IMPL_TC_Foo_MyType_0 == 'F') \
&& (TC_IMPL_TC_Foo_MyType_1 == 'o') \
&& (TC_IMPL_TC_Foo_MyType_2 == 'o') \
) && !defined(TC_DEF_TC_Foo_MyType)
#define TC_DEF_TC_Foo_MyType 1
static const char *anon_subnames_array0[] = {"eins", "zwei"};
static ORBIT2_MAYBE_CONST CORBA_TypeCode anon_subtypes_array1[] = {(CORBA_TypeCode)&TC_CORBA_short_struct, (CORBA_TypeCode)&TC_CORBA_long_struct};
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_Foo_MyType_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_struct,
0,
0,
MAX (ORBIT_ALIGNOF_CORBA_LONG, MAX (ORBIT_ALIGNOF_CORBA_SHORT, 1)),
0,
2
,
(CORBA_TypeCode *)anon_subtypes_array1,
CORBA_OBJECT_NIL,
"MyType",
"IDL:Foo/MyType:1.0",
(char **)anon_subnames_array0,
NULL,
-1,
0,
0, 0
};
#endif
#if ( (TC_IMPL_TC_Foo_Bar_Baz_0 == 'F') \
&& (TC_IMPL_TC_Foo_Bar_Baz_1 == 'o') \
&& (TC_IMPL_TC_Foo_Bar_Baz_2 == 'o') \
) && !defined(TC_DEF_TC_Foo_Bar_Baz)
#define TC_DEF_TC_Foo_Bar_Baz 1
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_Foo_Bar_Baz_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_objref,
0,
0,
ORBIT_ALIGNOF_CORBA_POINTER,
0,
0
,
NULL,
CORBA_OBJECT_NIL,
"Baz",
"IDL:Foo/Bar/Baz:1.0",
NULL,
NULL,
-1,
0,
0, 0
};
#endif
#if ( (TC_IMPL_TC_Foo_Hello_MyException_0 == 'F') \
&& (TC_IMPL_TC_Foo_Hello_MyException_1 == 'o') \
&& (TC_IMPL_TC_Foo_Hello_MyException_2 == 'o') \
) && !defined(TC_DEF_TC_Foo_Hello_MyException)
#define TC_DEF_TC_Foo_Hello_MyException 1
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_Foo_Hello_MyException_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_except,
0,
0,
1,
0,
0
,
NULL,
CORBA_OBJECT_NIL,
"MyException",
"IDL:Foo/Hello/MyException:1.0",
NULL,
NULL,
-1,
0,
0, 0
};
#endif
#if ( (TC_IMPL_TC_Foo_Hello_MyLong_0 == 'F') \
&& (TC_IMPL_TC_Foo_Hello_MyLong_1 == 'o') \
&& (TC_IMPL_TC_Foo_Hello_MyLong_2 == 'o') \
) && !defined(TC_DEF_TC_Foo_Hello_MyLong)
#define TC_DEF_TC_Foo_Hello_MyLong 1
static ORBIT2_MAYBE_CONST CORBA_TypeCode anon_subtypes_array14[] = {(CORBA_TypeCode)&TC_CORBA_long_struct};
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_Foo_Hello_MyLong_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_alias,
0,
0,
ORBIT_ALIGNOF_CORBA_LONG,
0,
1
,
(CORBA_TypeCode *)anon_subtypes_array14,
CORBA_OBJECT_NIL,
"MyLong",
"IDL:Foo/Hello/MyLong:1.0",
NULL,
NULL,
-1,
0,
0, 0
};
#endif
#if ( (TC_IMPL_TC_Foo_Hello_0 == 'F') \
&& (TC_IMPL_TC_Foo_Hello_1 == 'o') \
&& (TC_IMPL_TC_Foo_Hello_2 == 'o') \
) && !defined(TC_DEF_TC_Foo_Hello)
#define TC_DEF_TC_Foo_Hello 1
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_Foo_Hello_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_objref,
0,
0,
ORBIT_ALIGNOF_CORBA_POINTER,
0,
0
,
NULL,
CORBA_OBJECT_NIL,
"Hello",
"IDL:Foo/Hello:1.0",
NULL,
NULL,
-1,
0,
0, 0
};
#endif
#if ( (TC_IMPL_TC_Foo_MaximumHello_0 == 'F') \
&& (TC_IMPL_TC_Foo_MaximumHello_1 == 'o') \
&& (TC_IMPL_TC_Foo_MaximumHello_2 == 'o') \
) && !defined(TC_DEF_TC_Foo_MaximumHello)
#define TC_DEF_TC_Foo_MaximumHello 1
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_Foo_MaximumHello_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_objref,
0,
0,
ORBIT_ALIGNOF_CORBA_POINTER,
0,
0
,
NULL,
CORBA_OBJECT_NIL,
"MaximumHello",
"IDL:Foo/MaximumHello:1.0",
NULL,
NULL,
-1,
0,
0, 0
};
#endif
#if ( (TC_IMPL_TC_Foo_InheritanceTest_0 == 'F') \
&& (TC_IMPL_TC_Foo_InheritanceTest_1 == 'o') \
&& (TC_IMPL_TC_Foo_InheritanceTest_2 == 'o') \
) && !defined(TC_DEF_TC_Foo_InheritanceTest)
#define TC_DEF_TC_Foo_InheritanceTest 1
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_Foo_InheritanceTest_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_objref,
0,
0,
ORBIT_ALIGNOF_CORBA_POINTER,
0,
0
,
NULL,
CORBA_OBJECT_NIL,
"InheritanceTest",
"IDL:Foo/InheritanceTest:1.0",
NULL,
NULL,
-1,
0,
0, 0
};
#endif

#ifndef ORBIT_IDL_C_IMODULE_Foo
CORBA_unsigned_long Foo_Bar_Baz__classid = 0;
#endif

#ifndef ORBIT_IDL_C_IMODULE_Foo
CORBA_unsigned_long Foo_Hello__classid = 0;
#endif

#ifndef ORBIT_IDL_C_IMODULE_Foo
CORBA_unsigned_long Foo_MaximumHello__classid = 0;
#endif

#ifndef ORBIT_IDL_C_IMODULE_Foo
CORBA_unsigned_long Foo_InheritanceTest__classid = 0;
#endif

/* Interface type data */

static ORBit_IArg Foo_Bar_Baz_haveFun__arginfo [] = {
	{ TC_CORBA_short,  ORBit_I_ARG_IN | ORBit_I_COMMON_FIXED_SIZE, "a" },
	{ TC_CORBA_boolean,  ORBit_I_ARG_IN | ORBit_I_COMMON_FIXED_SIZE, "b" }
};

#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IMethod Foo_Bar_Baz__imethods [] = {
	{
		{ 2, 2, Foo_Bar_Baz_haveFun__arginfo, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, "haveFun", 7,
		0
}
};static CORBA_string Foo_Bar_Baz__base_itypes[] = {
"IDL:omg.org/CORBA/Object:1.0"
};
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IInterface Foo_Bar_Baz__iinterface = {
TC_Foo_Bar_Baz,{1, 1, Foo_Bar_Baz__imethods, FALSE},
{1, 1, Foo_Bar_Baz__base_itypes, FALSE}
};


#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IMethod Foo_Hello__imethods [] = {
	{
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, "doHello", 7,
		0
}
};static CORBA_string Foo_Hello__base_itypes[] = {
"IDL:omg.org/CORBA/Object:1.0"
};
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IInterface Foo_Hello__iinterface = {
TC_Foo_Hello,{1, 1, Foo_Hello__imethods, FALSE},
{1, 1, Foo_Hello__base_itypes, FALSE}
};

static ORBit_IArg Foo_MaximumHello_haveFunWithAnArgument__arginfo [] = {
	{ TC_CORBA_long,  ORBit_I_ARG_IN | ORBit_I_COMMON_FIXED_SIZE, "argument" },
	{ TC_CORBA_short,  ORBit_I_ARG_IN | ORBit_I_COMMON_FIXED_SIZE, "foo" }
};

#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IMethod Foo_MaximumHello__imethods [] = {
	{
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, "haveMoreFun", 11,
		0
}
, 	{
		{ 2, 2, Foo_MaximumHello_haveFunWithAnArgument__arginfo, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, "haveFunWithAnArgument", 21,
		0
}
};static CORBA_string Foo_MaximumHello__base_itypes[] = {
"IDL:Foo/Bar/Baz:1.0",
"IDL:Foo/Hello:1.0",
"IDL:omg.org/CORBA/Object:1.0"
};
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IInterface Foo_MaximumHello__iinterface = {
TC_Foo_MaximumHello,{2, 2, Foo_MaximumHello__imethods, FALSE},
{3, 3, Foo_MaximumHello__base_itypes, FALSE}
};


#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IMethod Foo_InheritanceTest__imethods [] = {
	{
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, "theMegaTest", 11,
		0
}
};static CORBA_string Foo_InheritanceTest__base_itypes[] = {
"IDL:Foo/Bar/Baz:1.0",
"IDL:Foo/Hello:1.0",
"IDL:Foo/MaximumHello:1.0",
"IDL:omg.org/CORBA/Object:1.0"
};
#ifdef ORBIT_IDL_C_IMODULE_Foo
static
#endif
ORBit_IInterface Foo_InheritanceTest__iinterface = {
TC_Foo_InheritanceTest,{1, 1, Foo_InheritanceTest__imethods, FALSE},
{4, 4, Foo_InheritanceTest__base_itypes, FALSE}
};

