/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT_IDL_C_IMODULE_Foo

#include "Foo-common.c"

#include <orbit/orb-core/orbit-small.h>

static CORBA_TypeCode Foo__itypes[] = {
	TC_Foo_MyType,
	TC_Foo_Hello_MyException,
	TC_Foo_Hello_MyLong,
	NULL
};

static ORBit_IInterface *Foo__iinterfaces[] = {
	&Foo_Bar_Baz__iinterface,
	&Foo_Hello__iinterface,
	&Foo_MaximumHello__iinterface,
	&Foo_InheritanceTest__iinterface,
	NULL
};
ORBit_IModule orbit_imodule_data = {
   20,
   Foo__iinterfaces,
 { 3, 3, Foo__itypes, FALSE }
};

