/* guile-gnome
 * Copyright (C) 2004 Free Software Foundation, Inc.
 *
 * gnome-vfs-support.h:
 *
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License, or (at your option) any later version.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact:
 *
 * Free Software Foundation           Voice:  +1-617-542-5942
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
 * Boston, MA  02111-1307,  USA       gnu@gnu.org
 */

#include <libgnomevfs/gnome-vfs.h>
#include <libguile.h>


#define SCM_VPORTP(x) (!SCM_IMP (x) && (SCM_TYP16 (x) == scm_tc16_vport))
#define SCM_OPVPORTP(x) (SCM_VPORTP (x) && (SCM_CELL_WORD_0 (x) & SCM_OPN))
#define SCM_OPINVPORTP(x) (SCM_OPVPORTP (x) && (SCM_CELL_WORD_0 (x) & SCM_RDNG))
#define SCM_OPOUTVPORTP(x) (SCM_OPVPORTP (x) && (SCM_CELL_WORD_0 (x) & SCM_WRTNG))

SCM scm_gnome_vfs_handle_to_port (GnomeVFSHandle *handle, GnomeVFSOpenMode mode,
                                  const gchar *name);
GnomeVFSHandle *scm_gnome_vfs_port_to_handle (SCM port);
void scm_init_gnome_vfs_ports (void);

